/*
--------  Global State Variable ---------
*/
var processedItemTotal = 0;
var itemList = [];

/*
Create all the context menu items.
*/
browser.contextMenus.create({
  id: "upload-content",
  title: "Upload",
  contexts: ["image", "video", "audio"]
}, onCreated);

browser.contextMenus.create({
  id: "paste-selection",
  title: "Paste Selection",
  contexts: ["selection"]
}, onCreated);

browser.contextMenus.create({
  id: "shorten-url",
  title: "Shorten Url",
  contexts: ["link"]
}, onCreated);

/*
The click event listener, where we perform the appropriate action given the
ID of the menu item that was clicked.
*/
browser.contextMenus.onClicked.addListener(function(info, tab) {
  switch (info.menuItemId) {
    case "upload-content":
      // Get the media type
      var mediaType = info.mediaType;
      // Get the url for the source
      var srcUrl = info.srcUrl;
      // Upload the content
      uploadFromUrl(srcUrl);
      break;
    case "paste-selection":
      // Get the selected text
      var selectedText = info.selectionText;
      // Paste the current selection
      createPaste(selectedText);
      break;
    case "shorten-url":
      // Get the url 
      var fullUrl = info.linkUrl;
      // Shorten the url
      shortenUrl(fullUrl);
      break;
  }
});

// Handle messages to the background processor
browser.runtime.onMessage.addListener(processMessage);

function processMessage(msg, sender, sendResponse) {
  switch (msg.cmd) {
    case 'progress': // Get the progress of an item
      break;
    case 'add-item': //
      console.log('Added ' + msg.type + ' Item: ' + msg.value);
      break;
    case 'del-item':
      delProcessItem(msg.id);
      return true;
      break;
    case 'get-item':
      var foundItem = getProcessItem(msg.id);
      return foundItem;
      break;
    case 'get-all-items':
      return itemList;
      break;
    case 'upload-file':
      uploadFromUrl(msg.file);
      break;
  }
}

// Updates the process item list
function addProcessItem(type, data) {
  // Update the total process count
  processedItemTotal++;
  
  // Create a new object for this process and add it to the main list
  var item = { id: processedItemTotal, type: type, data: data };
  itemList.push(item);
  
  // Update Badge
  updateBadge();
}

function getProcessItem(id) {  
  // Get the item from the master list
  var foundItem = itemList.find(function(item) {
      return item.id === id;
  });
  return foundItem;
}

function delProcessItem(id) {
  // Remove the item from the master list
  itemList = itemList.filter(function(item) {
      return item.id !== id;
  });
  
  // Update Badge
  updateBadge();
}

function updateBadge() {
  if (itemList.length === 0) {
    browser.browserAction.setBadgeText({text: ''});
  }
  else {
    browser.browserAction.setBadgeText({text: itemList.length.toString()});
  }
}