/*
Called when the item has been created, or when creation failed due to an error.
We'll just log success/failure here.
*/
function onCreated(n) {
  if (browser.runtime.lastError) {
    console.log('Error: ' + browser.runtime.lastError);
  } else {
    console.log("Item created successfully");
  }
}

/*
Called when the item has been removed.
We'll just log success here.
*/
function onRemoved() {
  console.log("Item removed successfully");
}

function onOpened() {
  console.log('Options page opened');
}

/*
Called when there was a response.
We'll just log the response here.
*/
function onResponse(msg) {
  console.log('Response: ' + msg);
}

/*
Called when there was an error.
We'll just log the error here.
*/
function onError(error) {
  console.log('Error: ' + error);
}

function notify(title, message) {
  console.log(title + ": " + message);
  browser.notifications.create({
    "type": "basic",
    "iconUrl": browser.extension.getURL("images/favicon-48.png"),
    "title": title,
    "message": message
  });
}

function randomString(length, chars) {
    var mask = '';
    if (chars.indexOf('a') > -1) mask += 'abcdefghijklmnopqrstuvwxyz';
    if (chars.indexOf('A') > -1) mask += 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
    if (chars.indexOf('#') > -1) mask += '0123456789';
    if (chars.indexOf('!') > -1) mask += '~`!@#$%^&*()_+-={}[]:";\'<>?,./|\\';
    var result = '';
    for (var i = length; i > 0; --i) result += mask[Math.floor(Math.random() * mask.length)];
    return result;
}

function getFileExtension(fileName) {
    var index = fileName.lastIndexOf('.');
    return '.' + fileName.substr(index + 1);
}

function GenerateBlobURL(url) {
    var cachedBlob = null;
    jQuery.ajax({
        url: url,
        success: function (result) {
            var workerJSBlob = new Blob([result], {
                type: "text/javascript"
            });
            cachedBlob = window.URL.createObjectURL(workerJSBlob);
        },
        async: false
    });
    return cachedBlob;
}