$(document).ready(function () {  
  $('#settings').click(function () {
    var opening = browser.runtime.openOptionsPage();
    opening.then(onOpened, onError);
  });  
  
  $('#upload').click(function (e) {
    // Prompt for uploading a file
    $('#files').click();
  });
  
  // Handle the addition of files
  $('#files').change(function (e) {
    var files = e.target.files;
    for (var i = 0; i < files.length; i++) {
      var f = files[i];
      var fileUrl = window.URL.createObjectURL(f);
      browser.runtime.sendMessage({
          cmd: 'upload-file',
          file: fileUrl
      });
    }
    var fileObj = $('#files');
    fileObj.replaceWith(fileObj = fileObj.clone(true));
  });
});

// When loaded, call the background to add processes
var sendingMessage = browser.runtime.sendMessage({
    cmd: 'get-all-items'
});
sendingMessage.then(fillItems, onError);

function fillItems(items) {
  if (items == null || items.length == 0) {
    // no items available
  }
  else {
    // Add a div for each item
    items.forEach(function(item) {      
      fillItem(item);
    });
  }
}

function fillItem(item) {
  var itemDiv;
  
  if ($('#process-item-' + item.id).length) {
    // It already exists, so let's update it
    itemDiv = $('#process-item-' + item.id);
  }
  else {  
    // Create a new one depending on the type
    switch (item.type) {
      case 'shorten':
      case 'paste':
      case 'upload':  
        // Setup a new itemDiv
        itemDiv = $('#process-item-template').clone();
        itemDiv.attr('id', 'process-item-' + item.id);
        
        itemDiv.find('#copy').click(function () {
          copyTextToClipboard(item.id);
        });        
        
        itemDiv.find('#open').click(function () {
          openItem(item.id);
        });
        
        itemDiv.find('#close').click(function () {
          clearItem(item.id);
        });
        break;     
      case 'progress':
        // Setup a new itemDiv
        itemDiv = $('#process-item-progress-template').clone();
        itemDiv.attr('id', 'process-item-' + item.id);
        
        itemDiv.find('#close').click(function () {
          cancelItem(item.id);
        });
        break;
      case 'error':
        // Setup a new itemDiv
        itemDiv = $('#process-item-error-template').clone();
        itemDiv.attr('id', 'process-item-' + item.id);
        
        itemDiv.find('#close').click(function () {
          clearItem(item.id);
        });
        break;
    }
    
    // Add to the main div
    $('#process-list').append(itemDiv);
  }
  
  // Add the data info depending on the type
  switch (item.type) {
    case 'shorten':
      itemDiv.find('#title').html('Shortened Url');
      itemDiv.find('#url').html(item.data.shortUrl);
      break;
    case 'paste':
      itemDiv.find('#title').html('Paste');
      itemDiv.find('#url').html(item.data.url);
      break;
    case 'upload':
      itemDiv.find('#title').html('Upload');
      var fullUrl = item.data.url;
      var key = '';
      if (item.data.key != null) {
          key = item.data.key;
      }
      if (!item.data.saveKey) {
          fullUrl = fullUrl + '#' + key;
      }
      itemDiv.find('#url').html(fullUrl);
      break;
    case 'progress':
      itemDiv.find('#title').html('Processing ' + item.type + '...');
      itemDiv.find('#progress').html(item.data.progress);
      break;
    case 'error':
      itemDiv.find('#title').html('Error');
      itemDiv.find('#error').html(item.data.message);
      break;
  }
}

function cancelItem(id) {
  var msg = browser.runtime.sendMessage({ cmd: 'cancel-item', id: id });
  msg.then(function() {
    $('#process-item-' + id).remove();
  }, onError);
}

function clearItem(id) {
  var msg = browser.runtime.sendMessage({ cmd: 'del-item', id: id });
  msg.then(function() {
    $('#process-item-' + id).remove();
  }, onError);
}

function openItem(id) {  
  var itemDiv = $('#process-item-' + id);
  browser.tabs.create({ url: itemDiv.find('#url').html() });
}

function copyItem(id) {
  var itemDiv = $('#process-item-' + id);
  copyTextToClipboard(itemDiv.find('#url').html());
}

function copyTextToClipboard(text) {
  var copyFrom = document.createElement("textarea");
  copyFrom.textContent = text;
  var body = document.getElementsByTagName('body')[0];
  body.appendChild(copyFrom);
  copyFrom.select();
  document.execCommand('copy');
  body.removeChild(copyFrom);
}