function uploadFromUrl(contentUrl) {
  // Download the file
  var xhr = new XMLHttpRequest();
  xhr.open('GET', contentUrl);
  xhr.responseType = 'blob';
  xhr.onload = function() {    
    // Encrypt the downloaded file
    encryptFile(xhr.response, xhr.response.type, false, false, false, 256, 128, 1000, uploadFile);
  };
  xhr.onerror = function(err) {
    onError("Upload Error: " + err);
  };
  xhr.onprogress = function (data) {
      if (data.lengthComputable) {
          var percentComplete = parseInt(((data.loaded / data.total) * 100), 10);
          // display loaded progress
          console.log("Download Progress" + percentComplete + '% Downloaded');
      }
  }
  xhr.send();
}

// Function to encrypt a file, overide the file's data attribute with the encrypted value, and then call a callback function if supplied
function encryptFile(fileBlob, contentType, saveKey, serverSideEncrypt, genDelKey, keySize, blockSize, chunkSize, callback) {
    var encScriptSrc = browser.extension.getURL('scripts/EncryptionWorker.js');
    var aesScriptSrc = browser.extension.getURL('scripts/Crypto-js/crypto-js.min.js');
    
    // Start the file reader
    var reader = new FileReader();

    // When the file has been loaded, encrypt it
    reader.onload = (function (callback) {
        return function (e) {
            // Create random key and iv (divide size by 8 for character length)
            var keyStr = randomString((keySize / 8), '#aA');
            var ivStr = randomString((blockSize / 8), '#aA');

            // Encrypt on the server side if they ask for it
            if (serverSideEncrypt) {
                callback(e.target.result, keyStr, ivStr, keySize, blockSize, contentType, saveKey, serverSideEncrypt, genDelKey);
            }
            else {
                var worker = new Worker(encScriptSrc);

                worker.onmessage = function (e) {
                    switch (e.data.cmd) {
                        case 'progress':
                            var percentComplete = Math.round(e.data.processed * 100 / e.data.total);
                            // Display encryption progress
                            console.log("Upload Progress: " + percentComplete + '% Encrypted');
                            break;
                        case 'finish':
                            if (callback != null) {
                                // Finish 
                                callback(e.data.buffer, keyStr, ivStr, keySize, blockSize, contentType, saveKey, serverSideEncrypt, genDelKey);
                            }
                            break;
                    }
                };

                worker.onerror = function (err) {
                    // An error occured
                    /* NOTIFY SOMEONE */
                    onError("Upload Error: " + err.message);
                }

                // Execute worker with data
                var objData =
                    {
                        cmd: 'encrypt',
                        script: aesScriptSrc,
                        key: keyStr,
                        iv: ivStr,
                        chunkSize: chunkSize,
                        file: e.target.result
                    };
                worker.postMessage(objData, [objData.file]);
            }
        };
    })(callback);

    reader.onprogress = function (data) {
        if (data.lengthComputable) {
            var percentComplete = parseInt(((data.loaded / data.total) * 100), 10);
            // display loaded progress
            console.log("Upload Progress" + percentComplete + '% Loaded');
        }
    }

    // Start async read
    reader.readAsArrayBuffer(fileBlob);
}

function uploadFile(data, key, iv, keySize, blockSize, contentType, saveKey, serverSideEncrypt, genDelKey)
{
    // Get the upload url
    var uploadAPI = "https://api.teknik.io/v1/Upload";
    
    // Create a blog from the data
    var blob = new Blob([data]);
    
    // Create the form data to send
    var fd = new FormData();
    fd.append('file', blob);
    fd.append('contentType', contentType);
    fd.append('saveKey', saveKey);
    if (saveKey)
    {
        fd.append('key', key);
    }
    fd.append('keySize', keySize);
    fd.append('iv', iv);
    fd.append('blockSize', blockSize);
    fd.append('encrypt', serverSideEncrypt);
    fd.append('genDeletionKey', genDelKey);

    // Send the form data and report the results
    var xhr = new XMLHttpRequest();
    xhr.upload.addEventListener("progress", uploadProgress.bind(null), false);
    xhr.addEventListener("load", uploadComplete.bind(null, key, saveKey), false);
    xhr.addEventListener("error", uploadFailed.bind(null), false);
    xhr.addEventListener("abort", uploadCanceled.bind(null), false);
    xhr.open("POST", uploadAPI);
    xhr.send(fd);
}

function uploadProgress(evt) {
    if (evt.lengthComputable) {
        var percentComplete = Math.round(evt.loaded * 100 / evt.total);
        console.log("Upload Progress" + percentComplete + '% Uploaded');
    }
}

function uploadComplete(key, saveKey, evt) {
    obj = JSON.parse(evt.target.responseText);
    if (obj.result != null) {
        var fullUrl = obj.result.url;
        if (obj.result.key != null)
            key = obj.result.key;
        if (!saveKey) {
            fullUrl = fullUrl + '#' + key;
        }
        
        // Add the saveKey to the results
        obj.result.saveKey = saveKey;
        
        // Open a new tab to the upload
        browser.tabs.create({ url: fullUrl });
        
        // Add the completed item
        addProcessItem('upload', obj.result);
    }
    else
    {
        if (obj.error != null) {
            onError("Upload Error: " + obj.error.message);
        }
        else {
            onError('Upload Error: Unable to Upload File');
        }
    }
}

function uploadFailed(evt) {
    onError('Upload Error: Upload Failed');
}

function uploadCanceled(evt) {
    onError('Upload Error: Upload Canceled');
}